package com.ningdatech.pmapi.dashboard.manage;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import com.ningdatech.pmapi.common.constant.BizConst;
import com.ningdatech.pmapi.common.statemachine.contants.RegionContant;
import com.ningdatech.pmapi.dashboard.constant.DashboardConstant;
import com.ningdatech.pmapi.dashboard.model.vo.ExpertStatisticsVO;
import com.ningdatech.pmapi.dashboard.model.vo.FundStatisticsVO;
import com.ningdatech.pmapi.gov.model.entity.GovBizProjectApply;
import com.ningdatech.pmapi.gov.model.entity.GovBizProjectApprove;
import com.ningdatech.pmapi.gov.model.entity.GovBizProjectBaseinfo;
import com.ningdatech.pmapi.gov.model.entity.GovBizProjectProcure;
import com.ningdatech.pmapi.gov.service.IGovBizProjectApplyService;
import com.ningdatech.pmapi.gov.service.IGovBizProjectApproveService;
import com.ningdatech.pmapi.gov.service.IGovBizProjectBaseinfoService;
import com.ningdatech.pmapi.gov.service.IGovBizProjectProcureService;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectStatusEnum;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import com.ningdatech.pmapi.sys.model.req.NoticeListReq;
import com.ningdatech.pmapi.todocenter.model.req.ToBeProcessedReq;
import com.ningdatech.pmapi.todocenter.model.vo.TodoCenterStatisticsVO;
import com.ningdatech.pmapi.user.security.auth.model.UserInfoDetails;
import com.ningdatech.pmapi.user.util.LoginUserUtil;
import com.ningdatech.pmapi.workbench.converter.WorkbenchConverter;
import io.swagger.models.auth.In;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ForkJoinPool;
import java.util.stream.Collectors;

/**
 * @Classname FundStatisticsManage
 * @Description
 * @Date 2023/11/24 17:44
 * @Author PoffyZhang
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class FundStatisticsManage {
    private final IProjectService projectService;

    private final IGovBizProjectBaseinfoService baseinfoService;

    private final IGovBizProjectApplyService applyService;

    private final IGovBizProjectApproveService approveService;

    private final IGovBizProjectProcureService procureService;

    private Integer[] years = new Integer[]{2021,2022,2023,2024,2025};

    private Integer[] threeYears = new Integer[]{LocalDateTime.now().getYear()-2,
            LocalDateTime.now().getYear()-1,LocalDateTime.now().getYear()};

    /**
     * 项目统计
     * @param year
     * @return
     */
    public FundStatisticsVO fund(Integer year) {
        FundStatisticsVO res = new FundStatisticsVO();

        CompletableFuture.allOf(
                CompletableFuture.runAsync(() -> {
                    //选择的年份去计算的数据
                    yearCompleteAmount(res,year);
                }, ForkJoinPool.commonPool()),
                CompletableFuture.runAsync(() -> {
                    //近三年
                    threeYearsCompleteAmount(res);
                }, ForkJoinPool.commonPool()),
                CompletableFuture.runAsync(() -> {
                    //所有年 用于算累积
                    allYearsCompleteAmount(res);
                }, ForkJoinPool.commonPool())
        ).join();

        return res;
    }

    /**
     * 计算 所有年的
     * @param res
     */
    private void allYearsCompleteAmount(FundStatisticsVO res) {
        //1.申报项目 和 归集项目
        //申报项目
        List<Project> allProjects = projectService.list(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getNewest, Boolean.TRUE)
                .in(Project::getProjectYear, years));

        //项目归集
        List<GovBizProjectApply> allApplieYears = applyService.list(Wrappers.lambdaQuery(GovBizProjectApply.class)
                .in(GovBizProjectApply::getBaseProjSetYear, years));

        List<String> allProjIds = allApplieYears.stream().map(GovBizProjectApply::getBaseProjId)
                .collect(Collectors.toList());

        List<GovBizProjectBaseinfo> allBaseInfos = Lists.newArrayList();
        List<GovBizProjectApply> allApplies = Lists.newArrayList();
        List<GovBizProjectApprove> allApproves = Lists.newArrayList();
        List<GovBizProjectProcure> allProcures = Lists.newArrayList();

        if(CollUtil.isNotEmpty(allProjIds)){
            allBaseInfos = baseinfoService.list(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                    .eq(GovBizProjectBaseinfo::getDeleted, Boolean.FALSE)
                    .in(GovBizProjectBaseinfo::getBaseProjId,allProjIds));
            if(CollUtil.isNotEmpty(allBaseInfos)){
                List<String> allBaseProjIds = allBaseInfos.stream().map(GovBizProjectBaseinfo::getBaseProjId)
                        .collect(Collectors.toList());
                allApplies = applyService.list(Wrappers.lambdaQuery(GovBizProjectApply.class)
                        .in(GovBizProjectApply::getBaseProjId,allBaseProjIds));
                allApproves = approveService.list(Wrappers.lambdaQuery(GovBizProjectApprove.class)
                        .in(GovBizProjectApprove::getBaseProjId,allBaseProjIds));
                allProcures = procureService.list(Wrappers.lambdaQuery(GovBizProjectProcure.class)
                        .in(GovBizProjectProcure::getBaseProjId,allBaseProjIds));
            }
        }

        //2.4 累积 核减
        res.setAccumulatedDeductionTotalAmount(completeDeductionAmount(allProjects,allApplies,
                allApproves,allProcures));
    }

    /**
     * 计算核减资金的方法
     * @param projects
     * @param applies
     * @param approves
     * @param procures
     * @return
     */
    private BigDecimal completeDeductionAmount(List<Project> projects,
                           List<GovBizProjectApply> applies, List<GovBizProjectApprove> approves,
                                               List<GovBizProjectProcure> procures) {
        //首先完成年度计划 并且未注销的项目
        List<Project> normalProjects = projects.stream().filter(p -> {
            if (Objects.nonNull(p.getStatus()) &&
                    p.getStatus().compareTo(ProjectStatusEnum.BE_SUSPENDED.getCode()) > 0) {
                return Boolean.TRUE;
            }
            return Boolean.FALSE;
        }).collect(Collectors.toList());

        //申报的申报金额(申报项目:初步方案申报金额 项目归集:项目总投资)
        BigDecimal declaredAmountApprove = normalProjects.stream().map(p -> {
            BigDecimal declareAmount = Objects.nonNull(p.getDeclareAmount()) ?
                    p.getDeclareAmount() : BigDecimal.ZERO;
            return declareAmount;
        }).reduce(BigDecimal::add).get();
        BigDecimal declaredAmountApproveGov = applies.stream().map(g -> {
            BigDecimal totalAmount = Objects.nonNull(g.getBaseProjTotalAmount()) ? g.getBaseProjTotalAmount() : BigDecimal.ZERO;
            return totalAmount;
        }).reduce(BigDecimal::add).get();
        BigDecimal declaredAmountApproveTotal = declaredAmountApprove.add(declaredAmountApproveGov);

        //申报的批复金额(申报项目:建设方案申报金额 项目归集:建议总投资)
        BigDecimal approvalAmountApprove = normalProjects.stream().map(p -> {
            BigDecimal declareAmount = Objects.nonNull(p.getApprovalAmount()) ? p.getApprovalAmount() : BigDecimal.ZERO;
            return declareAmount;
        }).reduce(BigDecimal::add).get();
        BigDecimal approvalAmountApproveGov = approves.stream().map(p -> {
            BigDecimal expertTotalMoney = Objects.nonNull(p.getBaseExpertTotalMoney()) ? p.getBaseExpertTotalMoney() : BigDecimal.ZERO;
            return expertTotalMoney;
        }).reduce(BigDecimal::add).get();
        BigDecimal approvalAmountApproveTotal = approvalAmountApprove.add(approvalAmountApproveGov);
        //申报的差额
        BigDecimal subtractApprove = declaredAmountApproveTotal.subtract(approvalAmountApproveTotal);

        //批复的批复金额(申报项目:归集信息建议批复总投资 项目归集:建议批复总投资)
        BigDecimal approvalAmountApproval = normalProjects.stream().map(p -> {
            BigDecimal proposeTotalInvest = Objects.nonNull(p.getProposeTotalInvest()) ? p.getProposeTotalInvest() : BigDecimal.ZERO;
            return proposeTotalInvest;
        }).reduce(BigDecimal::add).get();
        BigDecimal approvalAmountApprovalGov = approves.stream().map(p -> {
            BigDecimal reviewTotalMoney = Objects.nonNull(p.getBaseInitialReviewTotalMoney()) ? p.getBaseInitialReviewTotalMoney() : BigDecimal.ZERO;
            return reviewTotalMoney;
        }).reduce(BigDecimal::add).get();
        BigDecimal approvalAmountApprovalTotal = approvalAmountApproval.add(approvalAmountApprovalGov);

        //批复的差额
        BigDecimal subtractApproval = approvalAmountApproveTotal.subtract(approvalAmountApprovalTotal);

        //合同的批复金额(申报项目:成交金额 项目归集:成交金额)
        BigDecimal approvalAmountContract = normalProjects.stream().map(p -> {
            BigDecimal transactionAmount = Objects.nonNull(p.getTransactionAmount()) ? p.getTransactionAmount() : BigDecimal.ZERO;
            return transactionAmount;
        }).reduce(BigDecimal::add).get();
        BigDecimal approvalAmountContractGov = procures.stream().map(p -> {
            BigDecimal purchaseAmount = Objects.nonNull(p.getBaseProjPurchaseAmount()) ?
                    p.getBaseProjPurchaseAmount() : BigDecimal.ZERO;
            return purchaseAmount;
        }).reduce(BigDecimal::add).get();
        BigDecimal approvalAmountContractTotal = approvalAmountContract.add(approvalAmountContractGov);
        //合同的差额
        BigDecimal subtractContract = approvalAmountApprovalTotal.subtract(approvalAmountContractTotal);

        return subtractApprove.add(subtractApproval).add(subtractContract);
    }

    /**
     * 近三年
     * @param res
     */
    private void threeYearsCompleteAmount(FundStatisticsVO res) {
        //申报项目
        List<Project> threeYearsProjects = projectService.list(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getNewest, Boolean.TRUE)
                .in(Project::getProjectYear, threeYears));
        //项目归集
        List<GovBizProjectApply> threeYearsApplieYears = applyService.list(Wrappers.lambdaQuery(GovBizProjectApply.class)
                .in(GovBizProjectApply::getBaseProjSetYear, threeYears));
        List<String> threeYearsProjIds = threeYearsApplieYears.stream().map(GovBizProjectApply::getBaseProjId)
                .collect(Collectors.toList());
        List<GovBizProjectBaseinfo> threeYearsBaseInfos = Lists.newArrayList();
        List<GovBizProjectApply> threeYearsApplies = Lists.newArrayList();
        List<GovBizProjectApprove> threeYearsApproves = Lists.newArrayList();
        List<GovBizProjectProcure> threeYearsProcures = Lists.newArrayList();

        if(CollUtil.isNotEmpty(threeYearsProjIds)){
            threeYearsBaseInfos = baseinfoService.list(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                    .eq(GovBizProjectBaseinfo::getDeleted, Boolean.FALSE)
                    .in(GovBizProjectBaseinfo::getBaseProjId,threeYearsProjIds));
            if(CollUtil.isNotEmpty(threeYearsBaseInfos)){
                List<String> threeYearsBaseProjIds = threeYearsBaseInfos.stream().map(GovBizProjectBaseinfo::getBaseProjId)
                        .collect(Collectors.toList());
                threeYearsApplies = applyService.list(Wrappers.lambdaQuery(GovBizProjectApply.class)
                        .in(GovBizProjectApply::getBaseProjId,threeYearsBaseProjIds));
                threeYearsApproves = approveService.list(Wrappers.lambdaQuery(GovBizProjectApprove.class)
                        .in(GovBizProjectApprove::getBaseProjId,threeYearsBaseProjIds));
                threeYearsProcures = procureService.list(Wrappers.lambdaQuery(GovBizProjectProcure.class)
                        .in(GovBizProjectProcure::getBaseProjId,threeYearsBaseProjIds));
            }
        }

        //2.4 近三年 核减
        res.setThreeYearsDeductionTotalAmount(completeDeductionAmount(threeYearsProjects,threeYearsApplies,
                threeYearsApproves,threeYearsProcures));

        //安全起见 再排序下
        Arrays.sort(threeYears);

        List<FundStatisticsVO.AmountData> threeYearsDeduction = Lists.newArrayList();
        for(Integer year : threeYears){
            FundStatisticsVO.AmountData amountData = new FundStatisticsVO.AmountData();
            //申报项目
            List<Project> thisYearsProjects = projectService.list(Wrappers.lambdaQuery(Project.class)
                    .eq(Project::getNewest, Boolean.TRUE)
                    .eq(Project::getProjectYear, year));
            //项目归集
            List<GovBizProjectApply> thisYearApplieYears = applyService.list(Wrappers.lambdaQuery(GovBizProjectApply.class)
                    .eq(GovBizProjectApply::getBaseProjSetYear, year));
            List<String> thisYearProjIds = thisYearApplieYears.stream().map(GovBizProjectApply::getBaseProjId)
                    .collect(Collectors.toList());
            List<GovBizProjectBaseinfo> thisYearBaseInfos = Lists.newArrayList();
            List<GovBizProjectApply> thisYearApplies = Lists.newArrayList();
            List<GovBizProjectApprove> thisYearApproves = Lists.newArrayList();
            List<GovBizProjectProcure> thisYearProcures = Lists.newArrayList();

            if(CollUtil.isNotEmpty(thisYearProjIds)){
                thisYearBaseInfos = baseinfoService.list(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                        .eq(GovBizProjectBaseinfo::getDeleted, Boolean.FALSE)
                        .in(GovBizProjectBaseinfo::getBaseProjId,threeYearsProjIds));
                if(CollUtil.isNotEmpty(thisYearBaseInfos)){
                    List<String> threeYearsBaseProjIds = threeYearsBaseInfos.stream().map(GovBizProjectBaseinfo::getBaseProjId)
                            .collect(Collectors.toList());
                    thisYearApplies = applyService.list(Wrappers.lambdaQuery(GovBizProjectApply.class)
                            .in(GovBizProjectApply::getBaseProjId,threeYearsBaseProjIds));
                    thisYearApproves = approveService.list(Wrappers.lambdaQuery(GovBizProjectApprove.class)
                            .in(GovBizProjectApprove::getBaseProjId,threeYearsBaseProjIds));
                    thisYearProcures = procureService.list(Wrappers.lambdaQuery(GovBizProjectProcure.class)
                            .in(GovBizProjectProcure::getBaseProjId,threeYearsBaseProjIds));
                }
            }
            amountData.setName(year.toString());
            amountData.setAmount(completeDeductionAmount(thisYearsProjects,thisYearApplies,
                    thisYearApproves,thisYearProcures));
            threeYearsDeduction.add(amountData);
        }
        res.setThreeYearsDeduction(threeYearsDeduction);
    }

    /**
     * 选择年份去计算
     * @param res
     */
    private void yearCompleteAmount(FundStatisticsVO res, Integer year) {
        //申报项目
        List<Project> projects = projectService.list(Wrappers.lambdaQuery(Project.class)
                .eq(Project::getNewest, Boolean.TRUE)
                .eq(Objects.nonNull(year), Project::getProjectYear, year)
                .in(Objects.isNull(year), Project::getProjectYear, years));
        //项目归集
        List<GovBizProjectApply> applieYears = applyService.list(Wrappers.lambdaQuery(GovBizProjectApply.class)
                .eq(Objects.nonNull(year), GovBizProjectApply::getBaseProjSetYear, year)
                .in(Objects.isNull(year), GovBizProjectApply::getBaseProjSetYear, years));
        List<String> yearProjIds = applieYears.stream().map(GovBizProjectApply::getBaseProjId)
                .collect(Collectors.toList());
        List<GovBizProjectBaseinfo> baseInfos = Lists.newArrayList();
        List<GovBizProjectApply> applies = Lists.newArrayList();
        List<GovBizProjectApprove> approves = Lists.newArrayList();
        List<GovBizProjectProcure> procures = Lists.newArrayList();
        if(CollUtil.isNotEmpty(yearProjIds)){
            baseInfos = baseinfoService.list(Wrappers.lambdaQuery(GovBizProjectBaseinfo.class)
                    .eq(GovBizProjectBaseinfo::getDeleted, Boolean.FALSE)
                    .in(GovBizProjectBaseinfo::getBaseProjId,yearProjIds));
            if(CollUtil.isNotEmpty(baseInfos)){
                List<String> baseProjIds = baseInfos.stream().map(GovBizProjectBaseinfo::getBaseProjId)
                        .collect(Collectors.toList());
                applies = applyService.list(Wrappers.lambdaQuery(GovBizProjectApply.class)
                        .in(GovBizProjectApply::getBaseProjId,baseProjIds));
                approves = approveService.list(Wrappers.lambdaQuery(GovBizProjectApprove.class)
                        .in(GovBizProjectApprove::getBaseProjId,baseProjIds));
                procures = procureService.list(Wrappers.lambdaQuery(GovBizProjectProcure.class)
                        .in(GovBizProjectProcure::getBaseProjId,baseProjIds));
            }
        }
        //2
        //2.1 总申请金额
        BigDecimal declaredTotalAmount = projects.stream()
                .map(p -> Objects.nonNull(p.getDeclareAmount()) ? p.getDeclareAmount() : BigDecimal.ZERO)
                .reduce(BigDecimal::add).get();
        BigDecimal declaredTotalAmountGov = applies.stream()
                .map(a -> Objects.nonNull(a.getBaseProjTotalAmount()) ? a.getBaseProjTotalAmount() : BigDecimal.ZERO)
                .reduce(BigDecimal::add).get();
        res.setDeclaredTotalAmount(declaredTotalAmount.add(declaredTotalAmountGov));
        //2.2 总批复金额
        BigDecimal approvalTotalAmount = projects.stream()
                .map(p -> Objects.nonNull(p.getApprovalAmount()) ? p.getApprovalAmount() : BigDecimal.ZERO)
                .reduce(BigDecimal::add).get();
        BigDecimal approvalTotalAmountGov = approves.stream()
                .map(a -> Objects.nonNull(a.getBaseExpertTotalMoney()) ? a.getBaseExpertTotalMoney() : BigDecimal.ZERO)
                .reduce(BigDecimal::add).get();
        res.setApprovalTotalAmount(approvalTotalAmount.add(approvalTotalAmountGov));
        //2.3 总合同金额
        BigDecimal contractTotalAmount = projects.stream()
                .map(p -> Objects.nonNull(p.getContractAmount()) ? p.getContractAmount() : BigDecimal.ZERO)
                .reduce(BigDecimal::add).get();
        BigDecimal contractTotalAmountGov = procures.stream()
                .map(p -> Objects.nonNull(p.getBaseProjPurchaseAmount()) ? p.getBaseProjPurchaseAmount() : BigDecimal.ZERO)
                .reduce(BigDecimal::add).get();
        res.setApprovalTotalAmount(contractTotalAmount.add(contractTotalAmountGov));
    }

    /**
     * 专家统计
     * @param year
     * @return
     */
    public ExpertStatisticsVO expert(Integer year) {
        ExpertStatisticsVO res = new ExpertStatisticsVO();

        return res;
    }
}
